"""
Hipchat API
"""
import datetime

import requests

from zenoss_hipchat import config


class HipChatEventSendException(Exception):
    """
    Exception if the send failed
    """
    pass


class HipChatEvent(object):
    """
    Simple class for sending zenoss events to hipchat
    """

    SEVERITY_MAP = [
        ('green', 'Clear'),
        ('gray', 'Debug'),
        ('purple', 'Info'),
        ('yellow', 'Warn'),
        ('yellow', 'Error'),
        ('red', 'Crit'),
    ]

    NOTIFY_SEVERITY = config.NOTIFY_SEVERITY

    def __init__(
            self, device, info, component, severity,
            url, message, cleared_by, clear=False
    ):
        """
        Setup session and properties
        """
        self.device = device
        self.info = info
        self.component = component
        self.severity = severity
        self.url = url
        self.message = message
        self.cleared_by = cleared_by
        self.clear = clear
        self.time = datetime.datetime.now().isoformat()

        # Setup session
        self.session = requests.Session()
        self.session.headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        self.session.params = {
            'format': 'json',
            'auth_token': config.HIPCHAT_API_V1_TOKEN
        }
        self.post_url = 'https://{0}/v1/rooms/message'.format(
            config.HIPCHAT_API_V1_ENDPOINT
        )

    def _event_message(self):
        """
        Return templated event notification
        """
        return ('{device}: <a href="{url}">{info}</a><br />'
                '<b>Component</b>: {component}<br />'
                '<b>Time</b>: {time}<br />'
                '<b>Message</b>:'
                '{message}').format(**vars(self))

    def _clear_message(self):
        """
        Return templated clear notification
        """
        return ('<b><i>Cleared!</b></i><br />'
                '{device}:<a href="{url}">{info}</a> Cleared<br />'
                '<b>Cleared By</b>: {cleared_by}<br />'
                '<b>Component</b>: {component}<br />'
                '<b>Time</b>: {time}<br />'
                '<b>Message</b>:'
                '{message}').format(**vars(self))

    def _should_notify(self):
        """
        Determine if we should set the notify flag in HipChat call
        """
        if self.severity >= self.NOTIFY_SEVERITY:
            return 1
        return 0

    def send(self):
        """
        Send off the message to environment room with a nice template
        """

        if not self.clear:
            message = self._event_message()
        else:
            message = self._clear_message()

        from_name = "{0} ({1})".format(
            config.HIPCHAT_FROM, self.SEVERITY_MAP[self.severity][1]
        )

        response = self.session.post(
            url=self.post_url,
            data={
                'room_id': config.HIPCHAT_ROOM_ID,
                'from': from_name,
                'message_format': 'html',
                'notify': self._should_notify(),
                'color': self.SEVERITY_MAP[self.severity][0],
                'message': message
            },
            timeout=config.REQUEST_TIMEOUT
        )
        if response.status_code != 200:
            raise HipChatEventSendException(response.text)



"""
Python command to generate hipchat notification from Zenoss
events.
"""
from __future__ import print_function
import argparse
import sys


from zenoss_hipchat import config
from zenoss_hipchat.hipchat import HipChatEvent


def entry_point():
    """
    Entry point to script.
    """

    parser = argparse.ArgumentParser(
        prog='zenoss_hipchat',
        description=('Converts arguments into templated strings and uses '
                     'HipChat APIv1 to send the event to a HipChat room '
                     'specified by the environment variable HIPCHAT_ROOM '
                     'and the API token in the environment variable '
                     'HIPCHAT_TOKEN')
    )

    # Add required arguments
    req_opts = parser.add_argument_group('required arguments')
    req_opts.add_argument('-d', '--device', type=str, required=True,
                          help='Device where event occurred')
    req_opts.add_argument('-i', '--info', type=str, required=True,
                          help='Short message for event')
    req_opts.add_argument('-c', '--component', type=str, required=True,
                          help='Component of device for event')
    req_opts.add_argument('-s', '--severity', type=int, required=True,
                          help='Severity number from 0-5. See '
                          'http://community.zenoss.org/docs/DOC-9437#d0e6134')
    req_opts.add_argument('-u', '--url', type=str, required=True,
                          help='URL to go to event notificaiton')
    req_opts.add_argument('-m', '--message', type=str, required=True,
                          help='Long event message')

    # Optional arguments
    parser.add_argument('-b', '--cleared-by', type=str,
                        help='What cleared the event (when --clear is set)')
    parser.add_argument('-o', '--clear', action="store_true",
                        help="Set if event is being cleared")

    args = parser.parse_args()

    if not config.HIPCHAT_API_V1_TOKEN and not config.HIPCHAT_ROOM_ID:
        print('Environment variable "HIPCHAT_TOKEN" and "HIPCHAT_ROOM" '
              'must be specified and valid before this command can be run')
        sys.exit(-1)

    if args.clear and not args.cleared_by:
        print('--cleared-by is required when using --clear')
        sys.exit(-1)

    hip_chat_event = HipChatEvent(**vars(args))
    hip_chat_event.send()



"""
Simple environment variable based configuration for now
"""

import os


HIPCHAT_API_V1_TOKEN = os.environ.get('HIPCHAT_TOKEN', None)
HIPCHAT_ROOM_ID = os.environ.get('HIPCHAT_ROOM', None)
HIPCHAT_API_V1_ENDPOINT = os.environ.get(
    'HIPCHAT_API_ENDPOINT',
    'api.hipchat.com'
)
HIPCHAT_FROM = os.environ.get('HIPCHAT_FROM', 'Zenoss')
REQUEST_TIMEOUT = os.environ.get('HIPCHAT_TIMEOUT', 3)
NOTIFY_SEVERITY = os.environ.get('HIPCHAT_NOTIFY_SEVERITY', 4)



#!/usr/bin/env python
"""
Package installer for the zenoss-hipchat command
"""

from setuptools import setup, find_packages


VERSION = __import__('zenoss_hipchat').VERSION

with open('README.rst') as readme:
    README = readme.read()

setup(
    name='zenoss-hipchat',
    version=VERSION,
    packages=find_packages(),
    package_data={},
    license='GPLv3',
    author='Carson Gee',
    author_email='x@carsongee.com',
    url="http://github.com/carsongee/zenoss-hipchat",
    description=("Command suitable in use for Zenoss notification commands "
                 "for sending events to hipchat."),
    long_description=README,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Information Technology',
        ('License :: OSI Approved :: '
         'GNU Lesser General Public License v3 (LGPLv3)'),
        'Operating System :: POSIX :: Linux',
    ],
    install_requires=[
        'requests',
        ],
    entry_points={'console_scripts': [
        'zenoss-hipchat = zenoss_hipchat.command:entry_point',
    ]},
    zip_safe=True,
)
